/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#ifndef ALLOCATIONALGORITHM_H_
#define ALLOCATIONALGORITHM_H_

#include <string>
#include <vector>
#include <list>
#include <map>

#include "LiveMap.h"
#include "Segment.h"
#include "OnHold.h"
#include "synchronize.h"

using namespace std;

class AllocationAlgorithm
{
public:
    AllocationAlgorithm(string name);
    virtual ~AllocationAlgorithm() {}

//    void add_requester(char letter, Semaphore *sem);

    bool allocate(const string id, const int size);
    void deallocate(const string id);

    void finish();
    void print_statistics();

protected:
    list<Segment *> segment_list;
    list<Segment *>::iterator segment_list_iterator;

    virtual bool find_hole(const string id, const int size) = 0;

private:
    string name;

    map<string, Segment *> memory_map;
    map<char, Semaphore *> semaphore_map;
    vector<char> requester_vector;
    vector<OnHold> onhold_vector;
    LiveMap live_map;

    int request_count;
    int allocate_request_count;
    int deallocate_request_count;
    int onhold_count;
    int total_wait;

    int try_to_allocate(const string id, const int size);
    int insert_into_segment_list(const string id, const int size);
    void remove_from_segment_list(const string id);
    void print_segment_list();
    void put_request_on_hold(const string id, const int size);
    void process_allocations_on_hold();

    Semaphore *get_requester_semaphore(const string id);
};

#endif /* ALLOCATIONALGORITHM_H_ */
