/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <string>
#include <curses.h>
#include <stdlib.h>
#include <unistd.h>

#include "globals.h"
#include "LiveMap.h"

using namespace std;

void LiveMap::init()
{
    int x = 0, y = 0;

    initscr();
    cbreak();
    noecho();
    clear();

    for (int i = 0; i < MAX_BLOCKS; i++)
    {
        mvaddch(y, x, '.');

        if (++x == ROW_LENGTH)
        {
            x = 0;
            y++;
        }
    }

    refresh();
}

void LiveMap::paint_map(const char ch, const int size, const int n)
{
    for (int k = 0; k < size; k++)
    {
        int y = (n + k)/ROW_LENGTH;
        int x = (n + k)%ROW_LENGTH;

        mvaddch(y, x, ch);
    }

    refresh();
}

void LiveMap::pause()
{
    mvaddstr(line, 0, "Press any key to continue");

    refresh();
    getch();
}

void LiveMap::print()
{
    line++;
}

void LiveMap::print(char *text)
{
    mvaddstr(line++, 0, text);
}

void LiveMap::finish()
{
    mvaddstr(line, 0, "Press any key to quit");

    refresh();
    getch();
    endwin();
}
