/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <iostream>
#include "NextFit.h"

using namespace std;

NextFit::NextFit(string name) : AllocationAlgorithm(name)
{
    segment_list_iterator = segment_list.begin();
}

bool NextFit::find_hole(const string id, const int size)
{
    // Start searching from where the previous search ended
    // for any sufficiently large hole.
    auto starting_iterator = segment_list_iterator;

    do
    {
        Segment *segment = *segment_list_iterator;

        // Next fit search.
        if (segment->size < 0)
        {
            int extra = size + segment->size;
            if (extra <= 0) return true;  // found a hole big enough
        }

        // Increment the iterator and wrap around
        // if at the end of the segment list.
        if (++segment_list_iterator == segment_list.end())
        {
            segment_list_iterator = segment_list.begin();
        }
    } while (segment_list_iterator != starting_iterator);

    return false;
}
