/**
 * Simulation of memory management algorithms
 *
 * Ron Mak
 * Department of Computer Engineering
 * San Jose State University
 */
#include <iostream>
#include <string>
#include <vector>
//#include <map>
#include <fcntl.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>

#include "globals.h"
#include "synchronize.h"
#include "LiveMap.h"
#include "Request.h"
#include "Segment.h"
#include "AllocationAlgorithm.h"
#include "FirstFit.h"
#include "NextFit.h"
#include "BestFit.h"

void simulate(AllocationAlgorithm *alg);

int main(int argc, char *argv[])
{
    if (argc != 2)
    {
        fprintf(stderr, "Usage: id\n");
        return -1;
    }

    string option = argv[1];

    AllocationAlgorithm *alg = nullptr;

    if      (option == "-ff") alg = new FirstFit("First Fit Algorithm");
    else if (option == "-nf") alg = new NextFit("Next Fit Algorithm");
    else if (option == "-bf") alg = new BestFit("Best Fit Algorithm");
    else
    {
        cout << "Usage: MemoryManager -{ff, nf, bf}" << endl;
    }

//    int base_length = strlen(SEMAPHORE_NAME_BASE);
//    for (char letter = 'A'; letter <= 'C'; letter++)
//    {
//        Semaphore *sem = new Semaphore();
//        char *semaphore_name = (char *)malloc(base_length + 2);
//        sprintf(semaphore_name, "%s%c", SEMAPHORE_NAME_BASE, letter);
//
//        // Open an existing semaphore.
//        OPEN(sem, semaphore_name);
//        alg->add_requester(letter, sem);
//    }

    simulate(alg);
    return 0;
}

void simulate(AllocationAlgorithm *alg)
{
    // First open the FIFO in read-only mode
    // and make it nonblocking.
    int fd = open(FIFO_PATH, O_RDONLY);
    fcntl(fd, F_SETFL, O_NONBLOCK);

    Request req(fd);

    // Process requests from the FIFO.
    bool memory_full = false;
    while ((!memory_full) && req.next_request())
    {
        string id   = req.get_id();
        int    size = req.get_size();

        if (size > 0) memory_full = alg->allocate(id, size);
        else          alg->deallocate(id);
    }

    alg->print_statistics();
    alg->finish();
}
